;;; cperl-use-hash.el --- indent arguments to a "use" as hashes

;; Copyright 2008, 2009, 2010, 2011, 2013, 2014 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0 ...
;; Keywords: languages, perl
;; URL: http://user42.tuxfamily.org/cperl-use-hash/index.html
;; EmacsWiki: CPerlMode

;; cperl-use-hash.el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; cperl-use-hash.el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.


;;; Commentary:
;; 

;;; History:
;; 

;;; Code:

;; Explicit dependency on advice.el since `cperl-use-hash-unload-function'
;; needs `ad-find-advice' macro when running not byte compiled, and that
;; macro is not autoloaded.
(require 'advice)

(defadvice cperl-block-p (around cperl-use-hash activate)
  "\"use ... { ... }\" is a hash, not a block."
  ;; point is before {
  (if (save-excursion
        (cperl-backward-to-noncomment (point-min))
        (backward-word)
        (looking-at "sub\\|do\\b"))
      ;; "use foo sub {" or "do {" is a block
      (setq ad-return-value t)
    (if (save-excursion
          (cperl-backward-to-start-of-expr)
          (looking-at "use[^A-Za-z0-9:_]")) ;; no "[:alnum:]" in xemacs21
        ;; "use foo ... {" is a hash
        (setq ad-return-value nil)
      ad-do-it)))

(defun cperl-use-hash-unload-function ()
  "Remove advice on `cperl-block-p'.
This is called by `unload-feature'."
  (when (ad-find-advice 'cperl-block-p 'around 'cperl-use-hash)
    (ad-remove-advice   'cperl-block-p 'around 'cperl-use-hash)
    (ad-activate        'cperl-block-p))
  nil) ;; and do normal unload-feature actions too
 

;; (defadvice cperl-block-p (around cperl-do-anywhere activate)
;;   "\"do { ... }\" is a block anywhere"
;;   ;; point is before {
;;   (if (save-excursion
;;         (cperl-backward-to-start-of-expr)
;;         (looking-at "do[^A-Za-z0-9:_]")) ;; no "[:alnum:]" in xemacs21
;;       (setq ad-return-value t)
;;     ad-do-it))

;;         (and
;;              (progn
;;                (cperl-backward-to-noncomment (point-min))
;;                (backward-sexp)
;;                (looking-at "use\\>")))

(provide 'cperl-use-hash)

;;; cperl-use-hash.el ends here
