;; Copyright 2009, 2010, 2013 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(require 'compile)
(defun my-compilation-error-regexps ()
  (cond ((eval-when-compile (boundp 'compilation-error-regexp-systems-list))
         ;; xemacs21
         (mapcar (lambda (elem)
                   (cons (car elem)
                         (cadr elem)))
                 compilation-error-regexp-alist-alist))

        ((eval-when-compile (boundp 'compilation-error-regexp-alist-alist))
         ;; emacs22
         compilation-error-regexp-alist-alist)

        (t
         ;; emacs21
         (let ((i 0))
           (mapcar (lambda (elem)
                     (setq i (1+ i))
                     (cons (intern (format "entry-%d" i))
                           elem)))))))

(defun my-compilation-txt-section-list ()
  (save-excursion
    (re-search-backward "^symbols?: \\(.*\\)")
    (mapcar 'intern (split-string (match-string 1)))))

(defvar my-compilation-txt-filename "/down/emacs/etc/compilation.txt")
(setq my-compilation-txt-filename "my-compilation.txt")
(setq my-compilation-txt-filename "/tmp/x.txt")

(defun my-compilation-matches ()

  (let ((outbuf (get-buffer-create "*my-compilation-matches*"))
        (elem-list (my-compilation-error-regexps)))
    (with-current-buffer outbuf
      (erase-buffer))

    (let ((enable-local-variables nil))
      (find-file my-compilation-txt-filename)
      (save-excursion
        (goto-char (point-min))
        (if (re-search-forward "^* Directory tracking" nil t)
            (narrow-to-region (point-min) (point)))))

    ;; (insert (format "%S" (assoc 'absoft (my-compilation-error-regexps))))

    '(setq elem-list '(
                       (absoft "^\\(?:[Ee]rror on \\|[Ww]arning on\\( \\)\\)?[Ll]ine[ 	]+\\([0-9]+\\)[ 	]+of[ 	]+\"?\\([a-zA-Z]?:?[^\":
]+\\)\"?:" 3 2 nil (1))
                       ))


    (dolist (elem elem-list)
      (let ((name           (nth 0 elem))
            (regexp         (nth 1 elem))
            (filename-group (nth 2 elem))
            (linenum-group  (nth 3 elem)))

        ;;         (unless (string-match regexp "^\\^")
        ;;           (setq regexp (concat "^ *" regexp)))
        (goto-char (point-min))
        (while (re-search-forward regexp nil t)
          (let ((match        (match-string 0))
                (filename     (and filename-group
                                   (match-string filename-group)))
                (section-list (my-compilation-txt-section-list)))
            (unless (memq name section-list)
              (with-current-buffer outbuf
                (insert (format "%s: %S:\n  filename: %S\n  match: %S\n"
                                name
                                (if (= 1 (length section-list))
                                    (car section-list)
                                  section-list)
                                filename
                                match))))))))

    (switch-to-buffer outbuf)))


(add-to-list 'load-path (expand-file-name ".."))
(require 'compilation-perl)
(require 'compilation-weblint)
(require 'compilation-emacs21-batch)

(my-compilation-matches)


;; (setq r (cadr (assq 'makepp compilation-error-regexp-alist-alist)))
;; (setq compilation-error-regexp-alist '(makepp))
;; (setq compilation-error-regexp-alist '(gnu))
;; (setq compilation-error-regexp-alist '(ant))
;; (setq compilation-error-regexp-alist (mapcar 'car compilation-error-regexp-alist-alist))
;; (setq compilation-error-regexp-alist (reverse (mapcar 'car compilation-error-regexp-alist-alist)))
;; 
;; (car (my-compilation-error-regexps))
;; 
;;   (require 'compile)
;;   (let (ret)
;;     (save-excursion
;;       (goto-char (point-min))
;; 
;;       (if (fboundp 'compilation-parse-errors)
;; 	  (progn
;;             ;; emacs21, xemacs21
;; 
;; 	    (or compilation-error-list
;; 		(compilation-parse-errors (point-max) nil))
;; 	    (dolist (err (remq nil compilation-error-list))
;; 	      (let ((marker (car err))
;; 		    (lst    (cdr err)))
;; 		(my-push 'ret
;; 			 (list (my-line-number (marker-position marker))
;; 			       (car (nth 0 lst))  ;; filename
;; 			       (nth 1 lst))))))   ;; linenum
;; 
;;         ;; emacs22,emacs23
;; 
;;         ;; in emacs23 must force fontification to have the properties/markers
;;         ;; etc 
;;         (font-lock-fontify-buffer)
;; 	(let (locus)
;; 	  (while (setq locus
;; 		       (condition-case err
;; 			   (compilation-next-error 1)
;; 			 (error
;; 			  (if (equal err '(error "Moved past last error"))
;; 			      nil ;; stop loop
;; 			    ;; re-throw anything else
;; 			    (error "%s" (error-message-string err))))))
;; 
;; 	    (my-push 'ret (list (my-line-number)
;;                                 (caar (nth 2 (car locus)))
;;                                 (nth 1 (car locus))))))))
;; 
;;     (nreverse ret)))


;;-----------------------------------------------------------------------------
;; matching speed

;; each symbol
(progn
  (switch-to-buffer "x")
  (erase-buffer)
  (dolist (symbol (mapcar 'car compilation-error-regexp-alist-alist))
    (setq compilation-error-regexp-alist (list symbol))
    
    (let ((beg (float-time)))
      (find-file "/tmp/dist-deb.log")
      (let* ((end (float-time))
             (diff (- end beg)))
        (kill-buffer nil)
        (insert (format "%.3f %s\n" diff symbol))
        (redisplay t)))))

;; watcom slowness
(progn
  (switch-to-buffer "x")
  (erase-buffer)
  (dotimes (i 1000)
    (insert "xfoo barxx"))
  (insert "\n")

  (require 'compile)
  (setq compilation-error-regexp-alist '(watcom))
  (setq compilation-error-regexp-alist-alist
        '(
          (watcom
           "\\(\\(?:[a-zA-Z]:\\)?[^:(\t\n]+\\)(\\([0-9]+\\)): ?\
\\(?:\\(Error! E[0-9]+\\)\\|\\(Warning! W[0-9]+\\)\\):"
           1 2 nil (4))))

  (let ((beg (float-time)))
    (compilation-mode)
    (message "took %.2f" (- (float-time) beg))))

(find-file "/down/emacs/etc/compilation.txt")


(defun my-long ()
  (interactive)
  (save-excursion
  (goto-char (point-min))
  (let ((ret 0))
    (while (= 0 (forward-line))
      (end-of-line)
      (if (> (current-column) 200)
          (setq ret (1+ ret))))
    ret)))
