;; Copyright 2009, 2010, 2014 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Look for apparent urls in docstrings which don't have URL `...' linking.


(defun my-docstring-urls ()

  (require 'info-xref)
  (pop-to-buffer info-xref-output-buffer t)
  (let ((info-xref-filename-heading "")
	(case-fold-search t))

    (info-xref-output "Checking docstring URLs ...\n")
    (with-temp-buffer
      (mapatoms
       (lambda (symbol)
	 (when (and (not (eq 'calendar-mode symbol))
		    (fboundp symbol)
		    (symbol-function symbol)) ;; not nil
	   (let ((str (condition-case nil
			  (documentation symbol)
			(error nil))))
	     (when str
	       (erase-buffer)
	       (insert str)
	       (goto-char (point-min))
	       (while (re-search-forward "\\(https?:\\|s?ftp:\\)" nil t)
		 (unless (save-excursion
			   (goto-char (match-beginning 0))
			   (backward-word)
			   (looking-at "URL[ \t\r\n]+`"))
		   (info-xref-output "%s\n  %s\n"
				     symbol (thing-at-point 'line))))))))))
    (info-xref-output "done\n")))

(my-load-all--autoloads)
(my-docstring-urls)


(dolist (elfile (directory-files "/down/ding/gnus/lisp"
                                 nil
                                 "\\`[^.].*\\.el\\'"))
  (let ((feature (intern (substring elfile 0 -3))))
    (unless (memq feature '(dgnushack))
      (message "%s" feature)
      (condition-case nil ;; avoid error from gnus-vm loading vm
          (require feature)
        (error nil)))))
