;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; Functions and values used in safe-local-variable properties.
;;
;; No library loading for this since all such settings should be in
;; loaddefs, otherwise they can't work before the mode or library loads.
;;

(require 'cl)

(defun my-safe-local-variables-list ()
  (let (ret)
    (mapatoms
     (lambda (symbol)
       (when (get symbol 'safe-local-variable)
         (push symbol ret))))))

(defun my-safe-autoloads ()
  (load (expand-file-name "my-load-all.el"))

  (setq warning-minimum-log-level :debug)
  (let ((hash (make-hash-table :test 'eq)))

    (mapatoms
     (lambda (symbol)
       (when (get symbol 'safe-local-variable)
         (puthash symbol t hash))))

    (message "initial %s symbols" (hash-table-count hash))

    (dolist (name (my-load-all--libraries-list))
    (message "load %s" name)
      (my-load-all--one-library name)

      (mapatoms
       (lambda (symbol)
         (unless (gethash symbol hash)
           (when (get symbol 'safe-local-variable)
             (puthash symbol t hash)

             (lwarn 'my-load-all 'error
                    "library %S adds %S"
                    name symbol))))))))


(byte-compile 'my-safe-autoloads)
(my-safe-autoloads)

;; (my-load-all--libraries)
;; (setq my-after (my-safe-local-variables-list))

;;        (unless (and (symbolp func)
;;                     (member func seen))
;;          (insert (format "%s -- in `%s'\n\n"
;;                          (pp-to-string func)
;;                          (symbol-name symbol)))
;;          (push func seen)))))
;;   (goto-char (point-min)))
