#!/usr/bin/perl -w

# Copyright 2015 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use FindBin;
use File::stat ();
use File::Basename 'dirname';
use File::Spec;
use Image::Base::GD;
use IPC::Run;
use List::Util 'max','min';
use POSIX 'ceil';

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }

# uncomment this to run the ### lines
# use Smart::Comments;


my $size = 16;

# PNG spec 11.3.4.2 suggests RFC822 (or rather RFC1123) for CreationTime
use constant STRFTIME_FORMAT_RFC822 => '%a, %d %b %Y %H:%M:%S %z';
  my $program_fullname = File::Spec->catfile($FindBin::Bin, $FindBin::Script);
  my $st = File::stat::stat($program_fullname)
    // die "$program_fullname: $!";
  my $program_mtime = $st->mtime;
  $ENV{'TZ'} = 'UTC+0';
  my $creation_time = POSIX::strftime(STRFTIME_FORMAT_RFC822,
                                      gmtime($program_mtime));
### $creation_time

{
  my $filename = '/tmp/scroll-1/favicon.png';
  my @array
    =(
      ". . . . . . . . . . . . . . . . ", # 1
      ". . . . . . . . . . . . . . . . ",
      ". . . . . . J . . . . . . . . . ",
      ". . . . . . J . . . . . . . . . ", # 4
      ". . . . . . J . . . . . . . . . ",
      ". . . . . . J . . . . . . . . . ",
      ". . . J . . J . . . . . . . . . ",
      ". . . J . . J . . . X . . . . . ", # 8
      ". . . . J J . . . . X . . . . . ",
      ". . . . . . . . . . X . . X . . ",
      ". . . . . . . . . . X . X . . . ",
      ". . . . . . . . . . X X . . . . ", # 12
      ". . . . . . . . . . X . X . . . ",
      ". . . . . . . . . . X . . X . . ",
      ". . . . . . . . . . X . . . X . ",
      ". . . . . . . . . . . . . . . . ", # 16
     );

  my $image = Image::Base::GD->new (-height => $size,
                                    -width  => $size);
  $image->rectangle(0,0, $size-1,$size-1, 'black');
  foreach my $y (0 .. $size-1) {
    foreach my $x (0 .. $size-1) {
      if (substr($array[$y], 2*$x, 1) eq 'J') {
        $image->xy($x,$y, 'white');
      }
      if (substr($array[$y], 2*$x, 1) eq 'X') {
        $image->xy($x,$y, 'orange');
      }
    }
  }
  mkdir dirname($filename);
  $image->save($filename);

  pngtextadd($filename, 'Author',    'Kevin Ryde');
  pngtextadd($filename, 'Generator', "$FindBin::Script");
  pngtextadd($filename, 'Title',     'scroll-1.el - icon');
  pngtextadd($filename, 'Creation Time', $creation_time);
  pngtextadd($filename, 'Copyright', 'Copyright 2015 Kevin Ryde');
  pngtextadd($filename, 'Homepage',  'http://user42.tuxfamily.org/scroll-1/index.html');

  print "uncompressed ",-s $filename,"\n";
  # IPC::Run::run(['optipng','-quiet','-o5',$filename]);
  # print "compressed   ",-s $filename,"\n";

  IPC::Run::run(['xzgv',$filename]);
  exit 0;
}
{
  my $filename = '/tmp/quick-yes/favicon.png';
  my @array
    =(
      ". . . . . . . . . . . . . . . . ", # 1
      ". . . . . . . . . . . . . . . . ",
      ". . . . . . . . . . . . . . . . ",
      ". . . . . . . . . . . . . . . . ", # 4
      ". X . . . X . . . . . . . . . . ",
      ". X X . X X . . . . . . . . . . ",
      ". X . X . X . . . . . X . . X . ",
      ". X . X . X . . . . . X . . X . ", # 8
      ". X . . . X . X X X . X . . X . ",
      ". X . . . X . . . . . . X X X . ",
      ". X . . . X . . . . . . . . X . ",
      ". X . . . X . . . . . X . . X . ", # 12
      ". . . . . . . . . . . . X X . . ",
      ". . . . . . . . . . . . . . . . ",
      ". . . . . . . . . . . . . . . . ",
      ". . . . . . . . . . . . . . . . ", # 16
     );

  my $image = Image::Base::GD->new (-height => $size,
                                    -width  => $size);
  $image->rectangle(0,0, $size-1,$size-1, 'black');
  foreach my $y (0 .. $size-1) {
    foreach my $x (0 .. $size-1) {
      if (substr($array[$y], 2*$x, 1) eq 'X') {
        $image->xy($x,$y, 'lightgreen');
      }
    }
  }
  mkdir dirname($filename);
  $image->save($filename);

  pngtextadd($filename, 'Author',    'Kevin Ryde');
  pngtextadd($filename, 'Generator', "$FindBin::Script");
  pngtextadd($filename, 'Title',     'quick-yes.el - icon');
  pngtextadd($filename, 'Creation Time', $creation_time);
  pngtextadd($filename, 'Copyright', 'Copyright 2015 Kevin Ryde');
  pngtextadd($filename, 'Homepage',  'http://user42.tuxfamily.org/quick-yes/index.html');

  print "uncompressed ",-s $filename,"\n";
  # IPC::Run::run(['optipng','-quiet','-o5',$filename]);
  # print "compressed   ",-s $filename,"\n";

  IPC::Run::run(['xzgv',$filename]);
}

exit 0;

# add text to the png image in $filename
sub pngtextadd {
  my ($filename, $keyword, $value) = @_;
  system('pngtextadd', "--keyword=$keyword", "--text=$value", $filename) == 0
    or die "system(pngtextadd)";
}
