#!/usr/bin/perl

# Copyright 2009, 2010, 2012, 2013 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# Usage: ./grep-emacs.pl
#

use strict;
use warnings;
use POSIX;
use Perl6::Slurp;
use Iterator::Simple qw(igrep);
use Iterator::Simple::Locate;

use lib::abs '.';
use MyStuff;
use MyUniqByInode;
use MyUniqByMD5;

# uncomment this to run the ### lines
# use Smart::Comments;

my $verbose = 0;

my $it = Iterator::Simple::Locate->new (globs => [ # '*.el',
                                                   # '*.el.gz',
                                                   '/usr/share/emacs/site-lisp/*.el',
                                                   '/home/*.el',
                                                 ]);
{
  my $uniq = MyUniqByInode->new;
  $it = igrep { $uniq->uniq($_) } $it;
}
{
  my $uniq = MyUniqByMD5->new;
  $it = igrep { $uniq->uniq_filename($_) } $it;
}
# $it = igrep { $_ =~ /site-lisp/ } $it;

while (my $filename = $it->next) {
  if ($verbose) { print "look at $filename\n"; }

  my $str;
  if (! eval { $str = Perl6::Slurp::slurp($filename,
                                          ($filename =~ /gz$/
                                           ? ':gzip'
                                           : ()),
                                         ); 1 }) {
    next if $! == ENOENT;
    print "Cannot read $filename: $!\n";
    next;
  }
  ### $str

  my @splits;
  while ($str =~ /^\(/mg) {
    push @splits, $-[0];
  }
  push @splits, length($str);

  foreach my $i (0 .. $#splits-1) {
    my $str_pos = $splits[$i];
    my $part = substr($str, $str_pos, $splits[$i+1]-$str_pos);
    ### $part
    next unless $part =~ /^\(define-derived-mode(.*)/m;
    my $mode = $1;

    # next unless $part =~ /\(interactive[^-]/;
    # next unless $part =~ /\(kill-all-local-variables/;

    # while ($part =~ /paragraph-s/g) {
    #   my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $-[0]+$str_pos);
    #   print "$filename:$line:$col: $mode\n";
    # }

    while ($part =~ /set-syntax/g) {
      my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $-[0]+$str_pos);
      print "$filename:$line:$col: $mode\n";
    }
  }
}

exit 0;
