#!/usr/bin/perl

# Copyright 2013 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# Usage: ./grep-simplify.pl
#

use strict;
use warnings;
use POSIX;
use Perl6::Slurp;
use Iterator::Simple qw(igrep);
use Iterator::Simple::Locate;

use lib::abs '.';
use MyStuff;
use MyUniqByInode;
use MyUniqByMD5;

# uncomment this to run the ### lines
# use Smart::Comments;

my $verbose = 0;

my $it = Iterator::Simple::Locate->new (globs => [ '*.el',
                                                 #  '*.el.gz'
                                                 ]);
{
  my $uniq = MyUniqByInode->new;
  $it = igrep { $uniq->uniq($_) } $it;
}
{
  my $uniq = MyUniqByMD5->new;
  $it = igrep { $uniq->uniq_filename($_) } $it;
}
$it = igrep { $_ =~ /site-lisp/ } $it;

while (my $filename = $it->next) {
  ### $filename
  if ($verbose) { print "look at $filename\n"; }

  my $str = slurp($filename);
  $str = zap_comments($str);
  ### $str

  # (forward-word 1) (backward-word 1)
  while ($str =~ /(\(\s*(forward|backward)-word\s+-?1\s*\))/g) {
    my $form = $1;
    my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $-[0]);
    print "$filename:$line:$col: $form\n";
  }
}

sub zap_comments {
  my ($str) = @_;
  $str =~ s/^(\s*;.*)/' ' x length($1)/mge;
  return $str;
}

sub zslurp {
  my ($filename) = @_;
  my $str;
  eval { $str = Perl6::Slurp::slurp(['gzip(auto)'=>1],$filename); 1 }
    or do {
      #         if ($! == POSIX::ENOENT) {
      #        }
      print "Cannot read $filename: $!\n";
      return;
    };
  return $str;
}

exit 0;
