;; Copyright 2014, 2016 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-ac-tex-ref--temp*"))
    (switch-to-buffer tempbuf)

    (require 'ac-tex-ref)

    ;;-------------------------------------------------------------------------
    ;; ac-tex-ref-beginning-of-cite-name-position

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (pos  (nth 1 elem))
                    (want (nth 2 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))
                (let ((got (ac-tex-ref-beginning-of-cite-name-position)))
                  (if got (setq got (- got (point-min))))
                  (unless (equal got want)
                    (save-excursion
                      (goto-char (point-max))
                      (insert "\n\n\n")
                      (insert (format "\n\nac-tex-ref-beginning-of-cite-name-position\ngot:  %s\nwant: %s\nat count %s"
                                      got want count)))
                    (error "ac-tex-ref-beginning-of-cite-name-position\ngot:  %s\nwant: %s\nat count %s"
                           got want count))))
              (setq count (1+ count)))

            '(
              ;; LaTeX
              ("\\cite{" 0 nil)
              ("\\cite{" 1 nil)
              ("\\cite{" 2 nil)
              ("\\cite{" 3 nil)
              ("\\cite{" 4 nil)
              ("\\cite{" 5 nil)
              ("\\cite{" 6 6)

              ))


    ;;-------------------------------------------------------------------------
    ;; ac-tex-ref-beginning-of-ref-name-position

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (pos  (nth 1 elem))
                    (want (nth 2 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))
                (let ((got (ac-tex-ref-beginning-of-ref-name-position)))
                  (if got (setq got (- got (point-min))))
                  (unless (equal got want)
                    (save-excursion
                      (goto-char (point-max))
                      (insert "\n\n\n")
                      (insert (format "\n\nac-tex-ref-beginning-of-ref-name-position\ngot:  %s\nwant: %s\nat count %s"
                                      got want count)))
                    (error "ac-tex-ref-beginning-of-ref-name-position\ngot:  %s\nwant: %s\nat count %s"
                           got want count))))
              (setq count (1+ count)))

            '(
              ;; LaTeX
              ("\\ref{" 0 nil)
              ("\\ref{" 1 nil)
              ("\\ref{" 2 nil)
              ("\\ref{" 3 nil)
              ("\\ref{" 4 nil)
              ("\\ref{" 5 5)
              ("\\pageref{" 9 9)
              ("\\indexentry{x" 13 12)
              ("\\glossaryentry{x" 16 15)

              ;; amsmath
              ("\\eqref{abc}" 10 7)

              ;; hyperef
              ;; /usr/share/doc/texlive-doc/latex/hyperref/manual.pdf
              ("\\ref*{" 6 6)
              ("\\pageref*{" 10 10)
              ("\\hyperref[name" 10 10)
              ("\\hyperref[name]{text}" 14 10)
              ("\\hyperref[name]{text}" 16 nil) ;; not in text part
              ("\\autoref{" 9 9)
              ("\\autoref*{" 10 10)
              ("\\autopageref{" 13 13)
              ("\\autopageref*{" 14 14)

              ;; cleveref
              ;; /usr/share/doc/texlive-doc/latex/cleveref/cleveref.pdf
              ("\\cref{x" 7 6)
              ("\\cref*{x" 8 7)
              ("\\Cref{x,y" 8 8)
              ("\\Cref{x,y" 9 8)
              ("\\Cref*{x,y" 7 7)
              ("\\cpageref{xyz" 12 10)
              ("\\Cpageref{xyz" 12 10)
              ("\\cpageref*{xyz" 13 11)
              ("\\Cpageref*{xyz" 13 11)
              ("\\crefrange{xyz" 14 11)
              ("\\Crefrange{x}{y" 14 14)
              ("\\crefrange*{x}{xyz" 18 15)
              ("\\Crefrange*{x}{xyz" 18 15)
              ("\\cpagerefrange*{x}{xyz" 22 19)
              ;;
              ("\\namecref{x" 11 10)
              ("\\nameCref{x" 11 10)
              ("\\nameCrefs{x" 12 11)
              ("\\lcnamecref{x" 13 12)
              ("\\lcnamecrefs{x" 14 13)

              ;; varioref
              ;; /usr/share/doc/texlive-doc/latex/tools/varioref.pdf
              ("\\Ref{x" 6 5)
              ("\\vref{x" 7 6)
              ("\\vref*{x" 8 7)
              ("\\Vref{x" 7 6)
              ("\\Vref*{x" 8 7)
              ("\\vpageref{x" 11 10)
              ("\\vpageref*{x" 12 11)
              ("\\Vpageref{x" 11 10)
              ("\\Vpageref*{x" 12 11)
              ("\\vrefrange{xyz" 14 11)
              ("\\Vrefrange{x}{y" 14 14)
              ("\\vrefrange*{x}{xyz" 18 15)
              ("\\Vrefrange*{x}{xyz" 18 15)
              ("\\vpagerefrange*{x}{xyz" 22 19)
              ;;
              ("\\fullref{" 9 9)
              ("\\fullref*{" 10 10)
              ("\\Fullref{" 9 9)
              ("\\Fullref*{" 10 10)

              ;; ntheorem
              ;; /usr/share/doc/texlive-doc/latex/ntheorem/ntheorem.pdf
              ("\\thref{abc}" 10 7)
              ))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    ;; couple of cycles
    (require 'ac-tex-ref)
    (unload-feature 'ac-tex-ref t)
    (require 'ac-tex-ref)
    (unload-feature 'ac-tex-ref t)
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
