;; Copyright 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    (if (not (locate-library "mine-sweeper"))
        (message "skip, mine-sweeper.el not available")

      (require 'mine-sweeper)

      ;;-----------------------------------------------------------------------
      ;; load

      (require 'mine-sweeper-auto)

      ;; no leftovers in emacs21 byte compiling defadvice
      (when (and (boundp 'byte-compile-unresolved-functions)
                 byte-compile-unresolved-functions)
        (error "byte-compile-unresolved-functions after load: %S"
               byte-compile-unresolved-functions))
      (setq count (1+ count))


      ;;-----------------------------------------------------------------------
      ;; unload-feature

      (add-to-list 'load-path (expand-file-name "."))
      (require 'my-unload)

      (if (not my-unload-feature-function-p)
          (message "skip, no unload-feature function")

        (let ((feature 'mine-sweeper-auto))
          (unless (my-advice-by-name feature)
            (error "oops, didn't find any advice"))

          (unload-feature feature t)

          (let ((lst (my-advice-by-name feature)))
            (when lst
              (error "unload-feature leftover advice: %S" lst)))
          (setq count (1+ count))))

      ;;-----------------------------------------------------------------------
      ;; unload-feature -- with advice unloaded first

      (let ((feature 'mine-sweeper-auto))
        (if (featurep feature)
            (unload-feature feature t))
        (require feature)
        ;; if 'advice can be unloaded (no dependency expressed on it), then
        ;; see FEATURE unloads successfully too
        (when (condition-case nil
                  (progn (unload-feature 'advice) t)
                (error nil))
          (unload-feature feature t)))
      (setq count (1+ count))

      ;;-----------------------------------------------------------------------
      ;; customize

      (save-excursion
        (customize-group 'mine-sweeper-auto)
        (kill-buffer nil))


      ;;-----------------------------------------------------------------------
      (message "  %d tests" count)
      count)))

