;; Copyright 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(scroll-1-keybindings
              scroll-1-view-keybindings))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((compilation-mode-hook scroll-1-keybindings)
              (dictem-mode-hook      scroll-1-keybindings)
              (dictionary-mode-hook  scroll-1-keybindings)
              (Info-mode-hook        scroll-1-keybindings)
              (Man-mode-hook         scroll-1-keybindings)
              (view-mode-hook        scroll-1-view-keybindings)
              (w3m-mode-hook         scroll-1-keybindings)))

    ;;-------------------------------------------------------------------------
    ;; scroll-1-view-keybindings

    (scroll-1-view-keybindings)

    ;; eval-after-load
    (cond ((locate-library "view") ;; emacs
           (require 'view))
          ((locate-library "view-less") ;; xemacs21
           (require 'view-less))
          (t
           (message "oops, neither view.el nor view-less.el found")))
    (let ((got (lookup-key view-mode-map "j"))
          (want 'scroll-1-up))
      (unless (eq got want)
        (error "scroll-1-view-keybindings doesn't set `j'")))
    (setq count (1+ count))

    ;; when view-mode-map exists
    (define-key view-mode-map "j" nil)
    (scroll-1-view-keybindings)
    (let ((got (lookup-key view-mode-map "j"))
          (want 'scroll-1-up))
      (unless (eq got want)
        (error "scroll-1-view-keybindings doesn't re-set `j'")))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (message "  %d tests" count)
    count))
