;;; iswitchb-disfavour.el --- disfavour some buffers in iswitchb.

;; priority among disfavouring ?



;; Copyright 2008, 2009, 2010, 2011, 2013, 2014 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0 ...
;; Keywords: completion, convenience, iswitchb
;; URL: http://user42.tuxfamily.org/iswitchb-disfavour/index.html
;; EmacsWiki: CategoryBufferSwitching

;; iswitchb-disfavour.el is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; iswitchb-disfavour.el is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.


;;; Commentary:

;;; Install:

;; Put iswitchb-disfavour.el in one of your `load-path' directories and the
;; following in your .emacs
;;
;;     (autoload 'iswitchb-disfavour-reorder-buflist "iswitchb-disfavour")
;;     (add-hook 'iswitchb-make-buflist-hook
;;               'iswitchb-disfavour-reorder-buflist)
;;
;; There's a cookie below for the autoload, if you're brave enough to use
;; `update-file-autoloads' and friends.

;;; History:

;; Version 1 - the first version

;;; Code:

(eval-when-compile
  (unless (fboundp 'dolist)
    (require 'cl))) ;; for `dolist' macro in emacs20

;;;###autoload
(defgroup iswitchb-disfavour nil
  "iswitchb buffer name disfavouring."
 :prefix "iswitchb-disfavour-"
 :group 'iswitchb
 :link  '(url-link
          :tag "iswitchb-disfavour.el home page"
          "http://user42.tuxfamily.org/iswitchb-disfavour/index.html"))

(defcustom iswitchb-disfavour-buffer-list
  '("*Compile-Log*"
    "*Completions*"
    "TAGS")
  "List of buffer names to disfavour in iswitchb selection.
These buffers (those which exist) are moved to the end of the
selection list offered by `iswitchb-buffer' so as to
\"disfavour\" them and prefer other buffers in iswitchb's
matching.

Numbered buffers like \"TAGS<2>\" are disfavoured along with the
base name (those being usual for multiple buffers of the same
name, as per `generate-new-buffer-name')."

  :type  '(repeat string)
  :group 'iswitchb-disfavour)

(defcustom iswitchb-disfavour-buffer-regexps
  '("\\` "
    "\\.tex\\*\\'")
  "List of rexexps for names to disfavour in iswitchb selection.
Buffers with names matching any of these regexps are moved to the
end of the selection list offered by `iswitchb-buffer' so as to
\"disfavour\" them and prefer other buffers in iswitchb's
matching."

  :type '(repeat regexp)
  :group 'iswitchb-disfavour)

(defcustom iswitchb-disfavour-current-buffer
  nil
  "Whether to disfavour the current buffer in iswitchb selection."
  :type  'boolean
  :group 'iswitchb-disfavour)

;;;###autoload
(defun iswitchb-disfavour-reorder-buflist ()
  "Disfavour selected buffers for iswitchb selection.
This function is designed for use from `iswitchb-make-buflist-hook'.

It re-orders the buffer list iswitchb is building (in
`iswitchb-temp-buflist') so that the buffers of
`iswitchb-disfavour-buffer-list' are at the end.  The order
within the the \"disfavoured\" buffers is retained, with them all moved to the end of the list."

  (let* ((var    (if (boundp 'iswitchb-temp-buflist)
                     'iswitchb-temp-buflist ;; v1.5 emacs21,22
                   'buflist))               ;; v1.3 in xemacs21
         (regexp (concat "\\`\\("
                         (mapconcat 'regexp-quote
                                    iswitchb-disfavour-buffer-list
                                    "\\|")
                         "\\)\\(<[0-9]+>\\)?\\'"))
         (case-fold-search nil)
         good bad)

    (setq regexp (mapconcat (lambda (regexp)
                              (concat "\\(?:" regexp "\\)"))
                            (cons regexp iswitchb-disfavour-buffer-regexps)
                            "\\|"))

    (dolist (name (symbol-value var))
      (if (or (and iswitchb-disfavour-current-buffer
                   (equal name (buffer-name (current-buffer))))
              (string-match regexp name))
          (setq bad (cons name bad))
        (setq good (cons name good))))

    (set var (nconc good bad))))

;;;###autoload
(custom-add-option 'iswitchb-make-buflist-hook
                   'iswitchb-disfavour-reorder-buflist)

;; LocalWords: regexps

(provide 'iswitchb-disfavour)

;;; iswitchb-disfavour.el ends here
